.packageName <- "RcmdrPlugin.HH"
"BoxCox" <-
function(){
    require("car")
    initializeDialog(title="Box-Cox Transformations")
    variablesBox <- variableListBox(top, Numeric(), selectmode="multiple",
        title="Select variables (one or more)")
    onOK <- function(){
        variables <- getSelection(variablesBox)
        if (length(variables) < 1) {
            errorCondition(recall=BoxCox,
              message="You must select one or more variables.")
            return()
            }
        closeDialog()
        command <- paste("box.cox.powers(na.omit(cbind(",
            paste(paste(variables, "=", ActiveDataSet(), "$", variables, sep=""),
                collapse=", "), ")))", sep="")
        doItAndPrint(command)
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="box.cox.powers")
    tkgrid(getFrame(variablesBox), sticky="nw")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=2, columns=1)
    }

"DotplottbRcmdr" <-
function() {
    initializeDialog(title=gettextRcmdr("dotplot with tiebreakers"))
    factorBox <- variableListBox(top, Factors(), title=gettextRcmdr("Factors (pick zero or one)"), selectmode="single", initialSelection=-1)
    responseBox <- variableListBox(top, Numeric(), title=gettextRcmdr("~ Response Variable (pick one)"))
    groupBox <- variableListBox(top, Factors(), title=gettextRcmdr(" | Groups (pick zero or more)"), selectmode="multiple", initialSelection=-1)

    scalarsFrame <- tkframe(top)

    factor.jitterVar <- tclVar(".5")
    factor.jitterEntry <- tkentry(scalarsFrame, width="6", textvariable=factor.jitterVar)

    cexVar <- tclVar("1")
    cexEntry <- tkentry(scalarsFrame, width="6", textvariable=cexVar)

    pchVar <- tclVar("16")
    pchEntry <- tkentry(scalarsFrame, width="6", textvariable=pchVar)
    
    layxVar <- tclVar("")
    layxEntry <- tkentry(scalarsFrame, width="6", textvariable=layxVar)
    
    layyVar <- tclVar("")
    layyEntry <- tkentry(scalarsFrame, width="6", textvariable=layyVar)
    
    betxVar <- tclVar("0")
    betxEntry <- tkentry(scalarsFrame, width="6", textvariable=betxVar)
    
    betyVar <- tclVar("1")
    betyEntry <- tkentry(scalarsFrame, width="6", textvariable=betyVar)

    altxVar <- tclVar("1")
    altxEntry <- tkentry(scalarsFrame, width="6", textvariable=altxVar)
    
    altyVar <- tclVar("1")
    altyEntry <- tkentry(scalarsFrame, width="6", textvariable=altyVar)

    onOK <- function() {
        factor <- getSelection(factorBox)
        groups <- getSelection(groupBox)
        response <- getSelection(responseBox)

        factor.jitter  <- as.numeric(tclvalue(factor.jitterVar))
        cex            <- as.numeric(tclvalue(cexVar))
        old.warn <- options(warn=-1)
        pch            <- as.numeric(tclvalue(pchVar))
        if (is.na(pch)) pch <- paste("\'", tclvalue(pchVar), "\'", sep="")
        options(old.warn)
        layx           <- as.numeric(tclvalue(layxVar))
        layy           <- as.numeric(tclvalue(layyVar))
        betx           <- as.numeric(tclvalue(betxVar))
        bety           <- as.numeric(tclvalue(betyVar))
        altx           <- as.numeric(tclvalue(altxVar))
        alty           <- as.numeric(tclvalue(altyVar))
        
        closeDialog()
        if (0 == length(response)) {
            errorCondition(recall=DotplottbRcmdr,
                           message=gettextRcmdr("No response variable selected."))
            return()
            }
        .activeDataSet <- ActiveDataSet()
        groups.formula <- paste(groups, collapse=' + ')
        if (nchar(groups.formula) > 0) {
          groups.formula <- paste(" | ", groups.formula, sep="")
          freq.formula <- paste("with(get(.activeDataSet),",
                                "tapply(",
                                "get(response)",
                                ", data.frame(factor(get(response))",
                                if (length(factor)==1) ", get(factor)",
                                if (length(groups)>0) ", get(paste(groups, collapse=','))",
                                "), length))")
          freq <- eval(parse(text=freq.formula))
          max.freq <- max(freq, na.rm=TRUE)
          groups.formula <- paste(groups.formula,
                                  ", max.freq=", max.freq, sep="")
        }
        if (length(factor)==0) {
          n.response <- eval(parse(text='length(get(.activeDataSet)[[response]])'))
          factor <- paste("rep('', nrow(", .activeDataSet, "))", sep="")
        }
        dptb.command <- paste("dotplot(", factor, " ~ ",  response,
                              groups.formula,
                              ", data=", .activeDataSet, ",", sep="")

        if (is.na(layx) && !is.na(layy)) layx <- 1
        if (!is.na(layx) && is.na(layy)) layy <- 1
        if (is.na(layx) && is.na(layy))
          layout <- ""
        else
          layout <- paste(", layout=c(", layx, ",", layy, ")", sep="")

        between <- paste(", between=list(x=", betx, ",y=", bety, ")", sep="")
        alternating <- paste("scales=list(",
                             "x=list(alternating=", altx, "), ",
                             "y=list(alternating=", alty, "))", sep="")
        dptb.command <- paste(dptb.command,
                              " ", ## "\n        ",
                              "panel=panel.dotplot.tb",
                              ", factor=", factor.jitter,
                              ", cex=", cex,
                              ", pch=", pch,
                              layout,
                              between,
                              ", ", ## ",\n        ",
                              alternating,
                              ')',
                              sep="")
        doItAndPrint(dptb.command)
        activateMenus()
        tkfocus(CommanderWindow())
        }
    optionsFrame <- tkframe(top)
    buttonsFrame <- tkframe(top)
    OKCancelHelp(helpSubject="panel.dotplot.tb")
    tkgrid(getFrame(factorBox),
           getFrame(responseBox),
           getFrame(groupBox),
           sticky="nw")

    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("Change defaults if needed:")))
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("factor.jitter:")), factor.jitterEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("cex:")), cexEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("pch:")), pchEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("layx:")), layxEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("layy:")), layyEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("betx:")), betxEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("bety:")), betyEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("altx:")), altxEntry, sticky="w")
    tkgrid(tklabel(scalarsFrame, text=gettextRcmdr("alty:")), altyEntry, sticky="w")
    tkgrid(scalarsFrame, sticky="w")
    
    tkgrid(optionsFrame, columnspan=2, sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=3, columns=2)
  }
##source("~/HH-R.package/RcmdrPlugin.HH/R/Dotplot.tb.R")
##source("~/HH-R.package/HH/R/panel.dotplot.tb.R")
summary_HH <- function (object, ...) 
  UseMethod("summary_HH")

"summary_HH.regsubsets" <-
  function(object,
           names = abbreviate(dimnames(incidence)[[2]], minlength = abbrev),
           abbrev = 1, min.size = 1, max.size = dim(sumry$which)[2],
           statistic = c("bic", "cp", "adjr2", "rsq", "rss", "stderr"),
           las = par("las"), 
           cex.subsets = 1, ..., main=statistic) {

    x <- summary(object, ...)
    table.regsubsets <- as.data.frame(x[c("rsq","rss","adjr2","cp","bic")])
    table.regsubsets <- cbind(p=as.numeric(row.names(x$which))+1,
                              table.regsubsets)
    table.regsubsets$stderr <- sqrt(table.regsubsets$rss /
                                    (object$nn-table.regsubsets$p))
    sumry <- list(summary=table.regsubsets, which=x$which, nn=object$nn)
  
    incidence <- sumry$which
    incidence[] <- c(" ", "*")[1+sumry$which]
    if (dimnames(sumry$which)[[2]][1] == "(Intercept)")
      incidence <- incidence[,-1]
    statistic <- match.arg(statistic)
    stat <- switch(statistic,
                   bic = sumry$summary$bic,
                   cp = sumry$summary$cp, 
                   adjr2 = sumry$summary$adjr2,
                   rsq = sumry$summary$rsq,
                   rss = sumry$summary$rss,
                   stderr=sumry$summary$stderr)
    subset.size <- sumry$summary$p
    select <- subset.size >= min.size & subset.size <= max.size
    subset.size <- subset.size[select]
    stat <- stat[select]
    incidence <- incidence[select, ]
    abbrevs <- apply(incidence=="*", 1, function(x, names)
                     paste(names[x], sep="", collapse="-"),
                     names=names)
    
    model.names <- apply(incidence=="*", 1, function(x, names)
                         paste(names[x], sep="", collapse="-"),
                         names=dimnames(incidence)[[2]])
    
    .Summary <- cbind(model=abbrevs, sumry$summary)
    attr(.Summary, "abbrevs") <-
      data.frame(row.names=abbrevs, model=model.names, stringsAsFactors=FALSE)
    attr(.Summary, "n.max.adjr2") <- which.max(.Summary$adjr2)
    attr(.Summary, "n") <- sumry$nn
    class(.Summary) <- c("summary_HH_regsubsets", "data.frame")
    .Summary
}


plot.summary_HH_regsubsets <-
  function(x, ..., statistic="adjr2", legend=FALSE,
           col="darkgray", cex=1, pch=16,
           col.text="black", cex.text=1, col.abline="darkgray") {
    stat <- x[[statistic]]
    min.size <- min(stat)
    max.size <- max(stat)
    plot(x$p, stat,
         type = "p", xlab = "Number of Parameters", 
         ylab = paste("Statistic:", statistic), las=par("las"),
         ..., cex=cex, main=statistic, col=col, pch=pch,
         xlim=range(x$p)+c(-.4,.4))
    if (statistic == "cp") abline(a=1, b=1, col.abline=col.abline)
    
    for (i in seq(along = stat)) {
      adj <- if (x$p[i] == min.size) 
        0
      else if (x$p[i] == max.size) 
        1
      else 0.5
      text(x$p[i], stat[i], x$model[i],
           cex = cex.text, adj = adj, col=col.text)
    }
    
    if (legend) {
      abbrevs <- row.names(attr(x, "abbrevs"))
      legend(locator(1),
             legend = paste(format(abbrevs, justify="left"),
               attr(x, "abbrevs")$model, sep=" : "),
             adj=0)
    }
  }

"print.summary_HH_regsubsets" <-
function(x, ..., digits=3) {
  NextMethod("print", ..., digits=digits)
  cat('\nModel variables with abbreviations\n')
  print(attr(x,"abbrevs"))
  cat('\nmodel with largest adjr2\n')
  cat(attr(x,"n.max.adjr2"), "\n")
  cat('\nNumber of observations\n')
  cat(attr(x,"n"), "\n")
  invisible(x)
}
"Interaction2wtRcmdr" <-
function() {
    ## require("HH")
    if (length(grep("HH", search()))==0) stop("Please attach the HH directory.")
    initializeDialog(title=gettextRcmdr("Interaction twoway table"))
    groupBox <- variableListBox(top, Factors(), title=gettextRcmdr("Factors (pick two or more)"), selectmode="multiple")
    responseBox <- variableListBox(top, Numeric(), title=gettextRcmdr("Response Variable (pick one)"))
    onOK <- function() {
        groups <- getSelection(groupBox)
        response <- getSelection(responseBox)
        closeDialog()
        if (2 > length(groups)) {
            errorCondition(recall=Interaction2wtRcmdr, message=gettextRcmdr("Select at least two factors."))
            return()
            }
        if (0 == length(response)) {
            errorCondition(recall=Interaction2wtRcmdr, message=gettextRcmdr("No response variable selected."))
            return()
            }
        .activeDataSet <- ActiveDataSet()
        i2wt.command <- paste("interaction2wt(",
                              response, " ~ ", paste(groups, collapse=' + '),
                              ", data=", .activeDataSet, ')', sep="")
        doItAndPrint(i2wt.command)
        activateMenus()
        tkfocus(CommanderWindow())
        }
    optionsFrame <- tkframe(top)
    buttonsFrame <- tkframe(top)
    OKCancelHelp(helpSubject="interaction2wt")
    tkgrid(getFrame(groupBox), getFrame(responseBox), sticky="nw")
    tkgrid(optionsFrame, columnspan=2, sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=3, columns=2)
  }

"QQPlot.HH" <-
function()
  ## this function modified by Martin Maechler
  ## this function modified by Richard M. Heiberger to add Shapiro-Wilk test
{
    require("car")
    initializeDialog(title=gettextRcmdr("Quantile-Comparison (QQ) Plot"))
    xBox <- variableListBox(top, Numeric(), title=gettextRcmdr("Variable (pick one)"))
    onOK <- function(){
        x <- getSelection(xBox)
        closeDialog()
       if (0 == length(x)) {
            errorCondition(recall=QQPlot.HH, message=gettextRcmdr("You must select a variable.")) 
            return()
            }
        dist <- tclvalue(distVariable)
        save <- options(warn=-1)
        on.exit(options=save)
        retryMe <- function(msg) {
            Message(message= msg, type="error")
            QQPlot.HH()
        }
        switch(dist,
               "norm" = {
                 args <- 'dist= "norm"'
                 ShapWilk <- as.numeric(tclvalue(ShapWilkVariable))
               },
               "t" =  {
                   df <- tclvalue(tDfVariable)
                   df.num <- as.numeric(df)
                   if (is.na(df.num) || df.num < 1) {
                       retryMe(gettextRcmdr("df for t must be a positive number."))
                       return()
                   }
                   args <- paste('dist="t", df=', df, sep="")
               },
               "chisq" = {
                   df <- tclvalue(chisqDfVariable)
                   df.num <- as.numeric(df)
                   if (is.na(df.num) || df.num < 1) {
                       retryMe(gettextRcmdr("df for chi-square must be a positive number."))
                       return()
                   }
                   args <- paste('dist="chisq", df=', df, sep="")
               },
               "f" = {
                   df1 <- tclvalue(FDf1Variable)
                   df2 <- tclvalue(FDf2Variable)
                   df.num1 <- as.numeric(df1)
                   df.num2 <- as.numeric(df2)
                   if (is.na(df.num1) || df.num1 < 1 ||
                       is.na(df.num2) || df.num2 < 1) {
                       retryMe(gettextRcmdr("numerator and denominator \ndf for F must be positive numbers."))
                       return()                            
                   }
                   args <- paste('dist="f", df1=', df1, ', df2=', df2, sep="")
               },
               ## else -- other `dist' :
           {
               dist <- tclvalue(otherNameVariable)
               params <- tclvalue(otherParamsVariable)
               args <- paste('dist="', dist,'", ', params, sep="")
           }) # end{switch}
        .activeDataSet <- ActiveDataSet()
        labels <-
            if ("1" == tclvalue(identifyVariable))
                paste("rownames(", .activeDataSet, ")", sep="")
            else "FALSE"
        command <- paste("qq.plot", "(", .activeDataSet, "$", x, ", ", args,
                          ", labels=", labels, ")", sep="")
        command2 <- paste("shapiro.test", "(", .activeDataSet, "$", x, ")", sep="")
        doItAndPrint(command)
        if (dist=="norm" && ShapWilk) doItAndPrint(command2)
        activateMenus()
        tkfocus(CommanderWindow())
    }
    OKCancelHelp(helpSubject="qq.plot")
    distFrame <- tkframe(top)
    distVariable <- tclVar("norm")
    normalButton <- tkradiobutton(distFrame, variable=distVariable, value="norm")
    tButton <- tkradiobutton(distFrame, variable=distVariable, value="t")
    chisqButton <- tkradiobutton(distFrame, variable=distVariable, value="chisq")
    FButton <- tkradiobutton(distFrame, variable=distVariable, value="f")
    otherButton <- tkradiobutton(distFrame, variable=distVariable, value="other")
    checkBoxes(frame="normFrame", boxes=c("ShapWilk"), initialValues=c("1"),
        labels=gettextRcmdr(c("Shapiro-Wilk test of normality")))
    tDfFrame <- tkframe(distFrame)
    tDfVariable <- tclVar("")
    tDfField <- tkentry(tDfFrame, width="6", textvariable=tDfVariable)
    chisqDfFrame <- tkframe(distFrame)
    chisqDfVariable <- tclVar("")
    chisqDfField <- tkentry(chisqDfFrame, width="6", textvariable=chisqDfVariable)
    FDfFrame <- tkframe(distFrame)
    FDf1Variable <- tclVar("")
    FDf1Field <- tkentry(FDfFrame, width="6", textvariable=FDf1Variable)
    FDf2Variable <- tclVar("")
    FDf2Field <- tkentry(FDfFrame, width="6", textvariable=FDf2Variable)
    otherParamsFrame <- tkframe(distFrame)
    otherParamsVariable <- tclVar("")
    otherParamsField <- tkentry(otherParamsFrame, width="30", textvariable=otherParamsVariable)
    otherNameVariable <- tclVar("")
    otherNameField <- tkentry(otherParamsFrame, width="10", textvariable=otherNameVariable)
    identifyVariable <- tclVar("0")
    identifyFrame <- tkframe(top)
    identifyCheckBox <- tkcheckbutton(identifyFrame, variable=identifyVariable)
    tkgrid(getFrame(xBox), sticky="nw")
    tkgrid(tklabel(identifyFrame, text=gettextRcmdr("Identify observations with mouse")),
           identifyCheckBox, sticky="w")
    tkgrid(identifyFrame, sticky="w")
    tkgrid(tklabel(distFrame, text=gettextRcmdr("Distribution"), fg="blue"), columnspan=6, sticky="w")
    tkgrid(tklabel(distFrame, text=gettextRcmdr("Normal")), normalButton, normFrame, sticky="w")
    tkgrid(tklabel(tDfFrame, text=gettextRcmdr("df = ")), tDfField, sticky="w")
    tkgrid(tklabel(distFrame, text="t"), tButton, tDfFrame, sticky="w")
    tkgrid(tklabel(chisqDfFrame, text=gettextRcmdr("df = ")), chisqDfField, sticky="w")
    tkgrid(tklabel(distFrame, text=gettextRcmdr("Chi-square")), chisqButton,
           chisqDfFrame, sticky="w")
    tkgrid(tklabel(FDfFrame, text=gettextRcmdr("Numerator df = ")), FDf1Field,
           tklabel(FDfFrame, text=gettextRcmdr("Denominator df = ")), FDf2Field, sticky="w")
    tkgrid(tklabel(distFrame, text="F"), FButton, FDfFrame, sticky="w")
    tkgrid(tklabel(otherParamsFrame, text=gettextRcmdr("Specify: ")),
           otherNameField, tklabel(otherParamsFrame, text=gettextRcmdr("Parameters: ")),
           otherParamsField, sticky="w")
    tkgrid(tklabel(distFrame, text=gettextRcmdr("Other")), otherButton,
           otherParamsFrame, sticky="w")
    tkgrid(distFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=5, columns=1)
    }

"Scatter3DDialog.HH" <-
function(){
    use.rgl <- options("Rcmdr")[[1]]$use.rgl
    if (length(use.rgl) == 0 || use.rgl) require(rgl)    
    initializeDialog(title=gettextRcmdr("3D Scatterplot"))
    variablesFrame <- tkframe(top)
    .numeric <- Numeric()
    xBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("Explanatory variables (pick two)"), selectmode="multiple",
        initialSelection=NULL, listHeight=7)
    yBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("Response variable (pick one)"), listHeight=7)
    surfacesFrame <- tkframe(top)
    identifyPoints <- tclVar("0") 
    identifyPointsCheckBox <- tkcheckbutton(surfacesFrame, variable=identifyPoints)
    gridLines <- tclVar("1")
    gridLinesCheckBox <- tkcheckbutton(surfacesFrame, variable=gridLines)
    rglOpen <- tclVar("0")
    rglOpenCheckBox <- tkcheckbutton(surfacesFrame, variable=rglOpen)
    linearLSSurface <- tclVar("1")
    linearLSCheckBox <- tkcheckbutton(surfacesFrame, variable=linearLSSurface)
    betaMultiplierVariable <- tclVar(gettextRcmdr("1.0"))
    betaMultiplierField <- tkentry(surfacesFrame, width="6", textvariable=betaMultiplierVariable)
    quadLSSurface <- tclVar("0")
    quadLSCheckBox <- tkcheckbutton(surfacesFrame, variable=quadLSSurface)
    nonparSurface <- tclVar("0")
    nonparCheckBox <- tkcheckbutton(surfacesFrame, variable=nonparSurface)
    dfNonparVariable <- tclVar(gettextRcmdr("<auto>"))
    dfNonparField <- tkentry(surfacesFrame, width="6", textvariable=dfNonparVariable)
    additiveSurface <- tclVar("0")
    additiveCheckBox <- tkcheckbutton(surfacesFrame, variable=additiveSurface)
    dfAddVariable <- tclVar(gettextRcmdr("<auto>"))
    dfAddField <- tkentry(surfacesFrame, width="6", textvariable=dfAddVariable)
    bgFrame <- tkframe(top)
    bgVariable <- tclVar("white")
    whiteButton <- tkradiobutton(bgFrame, variable=bgVariable, value="white")
    blackButton <- tkradiobutton(bgFrame, variable=bgVariable, value="black")
    residFrame <- tkframe(top)
    residVariable <- tclVar("line")
    lineButton <- tkradiobutton(residFrame, variable=residVariable, value="line")
    squareButton <- tkradiobutton(residFrame, variable=residVariable, value="square")
    onOK <- function(){
        x <- getSelection(xBox)
        y <- getSelection(yBox)
        closeDialog()
        if (length(y) == 0) {
            errorCondition(recall=Scatter3DDialog.HH, message=gettextRcmdr("You must select a response variable."))
            return()
            }
        if (2 != length(x)) {
            errorCondition(recall=Scatter3DDialog.HH, message=gettextRcmdr("You must select 2 explanatory variables."))
            return()
            }
        if (is.element(y, x)) {
            errorCondition(recall=Scatter3DDialog.HH, message=gettextRcmdr("Response and explanatory variables must be different."))
            return()
            }
        grid <- if (tclvalue(gridLines) == 1) "TRUE" else "FALSE"
        lin <- if(tclvalue(linearLSSurface) == 1) '"linear"'
        quad <- if(tclvalue(quadLSSurface) == 1) '"quadratic"'
        nonpar <- if (tclvalue(nonparSurface) == 1) '"smooth"'
        additive <- if (tclvalue(additiveSurface) == 1) '"additive"'
        surfaces <- c(lin, quad, nonpar, additive)
        nsurfaces <- length(surfaces)
        opts <- options(warn=-1)
        betaMultiplier <- tclvalue(betaMultiplierVariable)
        betaMultiplier <- if (betaMultiplier == gettextRcmdr("1.0")) "" else paste(", coef.ratio=", as.numeric(betaMultiplier), sep="")
        dfNonpar <- tclvalue(dfNonparVariable)
        dfNonpar <- if (dfNonpar == gettextRcmdr("<auto>")) "" else paste(", df.smooth=", as.numeric(dfNonpar), sep="")
        dfAdd <- tclvalue(dfAddVariable)
        dfAdd <- if (dfAdd == gettextRcmdr("<auto>")) "" else paste(", df.additive=", as.numeric(dfAdd), sep="")
        options(opts)
        fit <- if (nsurfaces == 0) ", surface=FALSE"
            else if (nsurfaces == 1) paste(", fit=", surfaces, sep="")
            else paste(", fit=c(", paste(surfaces, collapse=","), ")", sep="")
        bg <- tclvalue(bgVariable)
        resid <- tclvalue(residVariable)
        .activeDataSet <- ActiveDataSet()
        if (.groups != FALSE){ 
            groups <- paste(", groups=", .activeDataSet, "$", .groups, sep="")
            parallel <- paste(", parallel=", .linesByGroup, sep="")
            }
        else groups <- parallel <- ""
        if (tclvalue(rglOpen) == 1) doItAndPrint("rgl.open()")
        command <- paste("scatter3d.HH(", .activeDataSet, "$", x[1], ", ", 
            .activeDataSet, "$", y, ", ", .activeDataSet, "$", x[2], fit, dfNonpar, 
            dfAdd, groups, parallel, ', bg="', bg, '", grid=', grid,
                         ', squares=', resid=="square", betaMultiplier,
            ', xlab="', x[1], '", ylab="', y, '", zlab="', x[2], '")', sep="")
        doItAndPrint(command)
        putRcmdr("rgl", TRUE)
        command <- paste("identify3d(", .activeDataSet, "$", x[1], ", ", 
            .activeDataSet, "$", y, ", ", .activeDataSet, "$", x[2], groups,
            ", labels=row.names(", .activeDataSet, "))", sep="")
        putRcmdr("Identify3d", command)
        .Tcl("update")
        if (tclvalue(identifyPoints) == 1) doItAndPrint(command)
        activateMenus()
        tkfocus(CommanderWindow())
        rgl.bringtotop()
        }
    groupsBox(Scatter3DDialog.HH, plotLinesByGroup=TRUE, plotLinesByGroupsText=gettextRcmdr("Parallel regression surfaces"))
    OKCancelHelp(helpSubject="Scatter3DDialog")
    tkgrid(getFrame(yBox), tklabel(variablesFrame, text="  "), getFrame(xBox), sticky="nw")
    tkgrid(variablesFrame, sticky="nw")
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Identify observations\nwith mouse")), identifyPointsCheckBox, sticky="w")
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Show surface grid lines")), gridLinesCheckBox, sticky="w")
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Open new 3D graphics window")), rglOpenCheckBox, sticky="w")
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Surfaces to Fit"), fg="blue"), sticky="w")

    betaMultiplierLabel <- tklabel(surfacesFrame, text=gettextRcmdr("coef multiple = "))
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Linear least-squares")), linearLSCheckBox, betaMultiplierLabel, betaMultiplierField, sticky="w")
    tkgrid.configure(betaMultiplierLabel, sticky="e")

    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Quadratic least-squares")), quadLSCheckBox, sticky="w")

    dfLabel <- tklabel(surfacesFrame, text=gettextRcmdr("df = "))
    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Smooth regression")), nonparCheckBox, 
        dfLabel, dfNonparField, sticky="w")
    tkgrid.configure(dfLabel, sticky="e")

    tkgrid(tklabel(surfacesFrame, text=gettextRcmdr("Additive regression")), additiveCheckBox, 
        tklabel(surfacesFrame, text=gettextRcmdr("df(each term) = ")), dfAddField, sticky="w")
    tkgrid(surfacesFrame, sticky="w") 
    tkgrid(tklabel(bgFrame, text=gettextRcmdr("Background Color"), fg="blue"), sticky="w", columnspan=2)
    tkgrid(tklabel(bgFrame, text=gettextRcmdr("Black")), blackButton, sticky="w")
    tkgrid(tklabel(bgFrame, text=gettextRcmdr("White")), whiteButton, sticky="w")
    tkgrid(bgFrame, sticky="w")

    tkgrid(tklabel(residFrame, text=gettextRcmdr("Residuals Display"), fg="blue"), sticky="w", columnspan=2)
    tkgrid(tklabel(residFrame, text=gettextRcmdr("Line")), lineButton, sticky="w")
    tkgrid(tklabel(residFrame, text=gettextRcmdr("Square")), squareButton, sticky="w")
    tkgrid(residFrame, sticky="w")

    tkgrid(groupsFrame, sticky="w")
    tkgrid(buttonsFrame, stick="w")
    dialogSuffix(rows=5, columns=1)
    }

"analyzeTwoWayTable.HH" <-
function(){
    initializeDialog(title=gettextRcmdr("Analyze Two-Way Table"))
    variablesFrame <- tkframe(top)
    onOK <- function() {
        rowPct <- tclvalue(rowPercentsVariable)
        colPct <- tclvalue(colPercentsVariable)
        totPct <- tclvalue(totPercentsVariable)

        chisq <- tclvalue(chisqTestVariable)
        chisqComp <- tclvalue(chisqCompVariable)
        chiComp <- tclvalue(chiCompVariable)
        expected <- tclvalue(expFreqVariable)
        fisher <- tclvalue(fisherTestVariable)
        closeDialog()

        ## .Table <- data.matrix(ActiveDataSet())
        command <- ActiveDataSet()
        logger(paste(".Table <- ", command, sep=""))
        assign(".Table", justDoIt(command), envir=.GlobalEnv)
        doItAndPrint(".Table")

        if (rowPct == 1) doItAndPrint("rowPercents(.Table) # Row Percentages")
        if (colPct == 1) doItAndPrint("colPercents(.Table) # Column Percentages")        
        if (totPct == 1) doItAndPrint("totPercents2.HH(.Table) # Total Percentages")        

        if (chisq == 1) {
            command <- "chisq.test(.Table, correct=FALSE)"
            logger(paste(".Test <- ", command, sep=""))
            assign(".Test", justDoIt(command), envir=.GlobalEnv)
            doItAndPrint(".Test")
            if (chisqComp == 1) doItAndPrint("round(.Test$residuals^2, 2) # Chi-square Components")
            if (chiComp == 1) doItAndPrint("round(.Test$residuals, 2) # Chi Components (residuals)")
            if (expected == 1) doItAndPrint("round(.Test$expected, 2) # Expected Counts")
            warnText <- NULL
            if (0 < (nlt1 <- sum(.Test$expected < 1))) warnText <- paste(nlt1,
                gettextRcmdr("expected frequencies are less than 1"))
            if (0 < (nlt5 <- sum(.Test$expected < 5))) warnText <- paste(warnText, "\n", nlt5,
                gettextRcmdr(" expected frequencies are less than 5"), sep="")
            if (!is.null(warnText)) Message(message=warnText,
                type="warning")
            logger("remove(.Test)") 
            remove(.Test, envir=.GlobalEnv) 
            }
        if (fisher == 1) doItAndPrint("fisher.test(.Table)")
        logger("remove(.Table)") 
        remove(.Table, envir=.GlobalEnv)                                                      
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="chisq.test")

    checkBoxes(frame="percentsFrame",
               boxes=c("rowPercents",
                 "colPercents",
                 "totPercents"), 
               initialValues=c("0","0","0"), 
               labels=gettextRcmdr(c(
                 "Row percentages",
                 "Column percentages",
                 "Total percentages")))

    checkBoxes(frame="testsFrame",
               boxes=c(
                 "chisqTest",
                 "chisqComp",
                 "chiComp",
                 "expFreq",
                 "fisherTest"),
               initialValues=c("1", "1", "0", "0", "0"),
               labels=gettextRcmdr(c(
                 "Chi-square test of independence",
                 "Print chi-square components",
                 "Print chi components (residuals)",
                 "Print expected frequencies",
                 "Fisher's exact test")))

    tkgrid(variablesFrame, sticky="w")
    tkgrid(tklabel(top, text=gettextRcmdr("Compute Percentages"), fg="blue"), sticky="w")
    tkgrid(percentsFrame, sticky="w")
    tkgrid(tklabel(top, text=gettextRcmdr("Hypothesis Tests"), fg="blue"), sticky="w")
    tkgrid(testsFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=4, columns=1)
  }

"anovaTableI.HH" <-
function(){
    .activeModel <- ActiveModel()
    if (!checkMethod("anova", .activeModel)) {
        errorCondition(message=gettextRcmdr("There is no appropriate anova method for a model of this class."))
        return()
        }
    doItAndPrint(paste("anova(", .activeModel, ")", sep=""))
    }

"bestSubsetsRegressionModel.HH" <-
function(){
    require("leaps")
    initializeDialog(title=gettextRcmdr("Best Subsets Regression"))
    variablesFrame <- tkframe(top)
    .numeric <- Numeric()
    xBox <- variableListBox(variablesFrame, .numeric, selectmode="multiple", 
        title=gettextRcmdr("Explanatory variables (pick one or more)"), listHeight=7)
    yBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("Response variable (pick one)"), listHeight=7)
    UpdateModelNumber()
    modelName <- tclVar(paste("RegModel.", getRcmdr("modelNumber"), sep=""))
    subsetsName <- tclVar(paste("Subsets.", getRcmdr("modelNumber"), sep=""))
    modelFrame <- tkframe(top)
    model <- tkentry(modelFrame, width="20", textvariable=subsetsName)
    subsetBox()
    nbestName <- tclVar("2")
    nbestFrame <- tkframe(top)
    nbest <- tkentry(nbestFrame, width="3", textvariable=nbestName)

    radioButtons(name="statistic",
                 buttons=c("rsq", "rss", "adjr2", "cp", "bic", "stderr"), 
                 values=c("rsq", "rss", "adjr2", "cp", "bic", "stderr"),
                 initialValue="adjr2", 
                 labels=gettextRcmdr(c("R Square", "Residual Sum of Squares",
                   "Adjusted R^2", "Cp", "BIC", "Standard Error")),
                 title=gettextRcmdr("Statistic to plot"))

    onOK <- function() {
        x <- getSelection(xBox)
        y <- getSelection(yBox)
        closeDialog()
        if (0 == length(y)) {
            UpdateModelNumber(-1)
            errorCondition(recall=bestSubsetsRegressionModel.HH, message=gettextRcmdr("You must select a response variable."))
            return()
            }
        if (0 == length(x)) {
            UpdateModelNumber(-1)
            errorCondition(recall=bestSubsetsRegressionModel.HH, message=gettextRcmdr("No explanatory variables selected."))
            return()
            }        
        if (is.element(y, x)) {
            UpdateModelNumber(-1)
            errorCondition(recall=bestSubsetsRegressionModel.HH, message=gettextRcmdr("Response and explanatory variables must be different."))
            return()
            }
        subset <- tclvalue(subsetVariable)
        if (trim.blanks(subset) == gettextRcmdr("<all valid cases>") || trim.blanks(subset) == ""){
            subset <- ""
            putRcmdr("modelWithSubset", FALSE)
            }
        else{
            subset <- paste(", subset=", subset, sep="")
            putRcmdr("modelWithSubset", TRUE)
            }
        subsetsValue <- trim.blanks(tclvalue(subsetsName))
        if (!is.valid.name(subsetsValue)){
            UpdateModelNumber(-1)
            errorCondition(recall=bestSubsetsRegressionModel.HH, message=sprintf(gettextRcmdr('"%s" is not a valid name.'), subsetsValue))
            return()
            }
        if (is.element(subsetsValue, listLinearModels())) {
            if ("no" == tclvalue(checkReplace(subsetsValue, type=gettextRcmdr("Model")))) {
                UpdateModelNumber(-1)
                bestSubsetsRegressionModel.HH()
                return()
                }
            }

        nbestValue <- as.integer(tclvalue(nbestName))
        if (!((is.integer(nbestValue)) && nbestValue>0 && nbestValue<=length(x))) {
            UpdateModelNumber(-1)
            errorCondition(recall=bestSubsetsRegressionModel.HH, message=sprintf(gettextRcmdr('"%s" is not a valid nbest value.'), nbestValue))
            return()
            }
        statisticValue <- tclvalue(statisticVariable)

        command <- paste("regsubsets(", y, "~", paste(x, collapse="+"),
                         ", data=", ActiveDataSet(), subset,
                         ", nbest=", nbestValue,
                         ")", sep="")
        logger(paste(subsetsValue, " <- ", command, sep=""))
        assign(subsetsValue, justDoIt(command), envir=.GlobalEnv)

        command <- paste("summary_HH(", subsetsValue, ")", sep="")
        summaryValue <- paste(subsetsValue, "Summary", sep=".")
        logger(paste(summaryValue, " <- ", command, sep=""))
        assign(summaryValue, justDoIt(command), envir=.GlobalEnv)
        ## summaries <-
        doItAndPrint(summaryValue)

        command <- paste("plot(", summaryValue, ", statistic='", statisticValue, "', legend=FALSE)", sep="")
        logger(command)
        justDoIt(command)
        
        bringToTop()
        .nmax <- attr(get(summaryValue, envir=.GlobalEnv), "n.max.adjr2")
        ## .nmax <- attr(summaries, "n.max.adjr2")
        modelValue <- paste(trim.blanks(tclvalue(modelName)), .nmax, sep=".")

        command <- paste("lm.regsubsets(", subsetsValue, ", ", .nmax, ")", sep="")
        logger(paste(modelValue, " <- ", command, "  ## subset ", .nmax, " has largest adjr2", sep=""))
        assign(modelValue , justDoIt(command), envir=.GlobalEnv)
        doItAndPrint(paste("summary(", modelValue, ")", sep=""))

        activeModel(modelValue)
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="regsubsets", model=TRUE)
    tkgrid(tklabel(modelFrame, text=gettextRcmdr("Enter name for model:")), model, sticky="w")
    tkgrid(modelFrame, sticky="w")
    tkgrid(getFrame(yBox), tklabel(variablesFrame, text="    "), getFrame(xBox), sticky="nw")
    tkgrid(variablesFrame, sticky="w")    

    tkgrid(tklabel(nbestFrame, text=gettextRcmdr("Number of subsets of each size to record:")), nbest, sticky="w")
    tkgrid(nbestFrame, sticky="w")

    tkgrid(statisticFrame, sticky="w")

    tkgrid(subsetFrame, sticky="w")
    tkgrid(buttonsFrame, stick="w")
    tkgrid.configure(helpButton, sticky="e")
    dialogSuffix(rows=5, columns=1)
    }

"confidenceIntervalsPlot" <-
function(){
  require("lattice")
  initializeDialog(title=gettextRcmdr("Confidence Intervals in Simple Linear Regression"))
    variablesFrame <- tkframe(top)
    .numeric <- Numeric()
    xBox <- variableListBox(variablesFrame, .numeric,
        title=gettextRcmdr("Explanatory variables (pick one)"))
    yBox <- variableListBox(variablesFrame, .numeric, title=gettextRcmdr("Response variable (pick one)"))
    UpdateModelNumber()
    modelName <- tclVar(paste("RegModel.", getRcmdr("modelNumber"), sep=""))
    modelFrame <- tkframe(top)
    model <- tkentry(modelFrame, width="20", textvariable=modelName)
    subsetBox()
    onOK <- function(){
        x <- getSelection(xBox)
        y <- getSelection(yBox)
        closeDialog()
        if (0 == length(y)) {
            UpdateModelNumber(-1)
            errorCondition(recall=confidenceIntervalsPlot, message=gettextRcmdr("You must select a response variable."))
            return()
            }
        if (0 == length(x)) {
            UpdateModelNumber(-1)
            errorCondition(recall=confidenceIntervalsPlot, message=gettextRcmdr("No explanatory variables selected."))
            return()
            }        
        if (is.element(y, x)) {
            UpdateModelNumber(-1)
            errorCondition(recall=confidenceIntervalsPlot, message=gettextRcmdr("Response and explanatory variables must be different."))
            return()
            }
        subset <- tclvalue(subsetVariable)
        if (trim.blanks(subset) == gettextRcmdr("<all valid cases>") || trim.blanks(subset) == ""){
            subset <- ""
            putRcmdr("modelWithSubset", FALSE)
            }
        else{
            subset <- paste(", subset=", subset, sep="")
            putRcmdr("modelWithSubset", TRUE)
            }
        modelValue <- trim.blanks(tclvalue(modelName))
        if (!is.valid.name(modelValue)){
            UpdateModelNumber(-1)
            errorCondition(recall=confidenceIntervalsPlot, message=sprintf(gettextRcmdr('"%s" is not a valid name.'), modelValue))
            return()
            }
        if (is.element(modelValue, listLinearModels())) {
            if ("no" == tclvalue(checkReplace(modelValue, type=gettextRcmdr("Model")))){
                UpdateModelNumber(-1)
                confidenceIntervalsPlot()
                return()
                }
            }
        command <- paste("lm(", y, "~", paste(x, collapse="+"),
            ", data=", ActiveDataSet(), subset, ")", sep="")
        logger(paste(modelValue, " <- ", command, sep=""))
        assign(modelValue, justDoIt(command), envir=.GlobalEnv)
        doItAndPrint(paste("summary(", modelValue, ")", sep=""))
        activeModel(modelValue)
        doItAndPrint(paste("ci.plot(", modelValue, ")", sep=""))
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="ci.plot", model=TRUE)
    tkgrid(tklabel(modelFrame, text=gettextRcmdr("Enter name for model:")), model, sticky="w")
    tkgrid(modelFrame, sticky="w")
    tkgrid(getFrame(yBox), tklabel(variablesFrame, text="    "), getFrame(xBox), sticky="nw")
    tkgrid(variablesFrame, sticky="w")    
    tkgrid(subsetFrame, sticky="w")
    tkgrid(buttonsFrame, stick="w")
    tkgrid.configure(helpButton, sticky="e")
    dialogSuffix(rows=4, columns=1)
    }

"enterTable.HH" <-
function(){
    require("abind")
    env <- environment()
    initializeDialog(title=gettextRcmdr("Enter Two-Way Table"))
    outerTableFrame <- tkframe(top)
    assign(".tableFrame", tkframe(outerTableFrame), envir=env)
    setUpTable <- function(...){
        tkdestroy(get(".tableFrame", envir=env))
        assign(".tableFrame", tkframe(outerTableFrame), envir=env)
        nrows <- as.numeric(tclvalue(rowsValue))
        ncols <- as.numeric(tclvalue(colsValue))
        make.col.names <- "tklabel(.tableFrame, text='')"
        for (j in 1:ncols) {
            col.varname <- paste(".colname.", j, sep="")
            assign(col.varname, tclVar(j), envir=env)
            make.col.names <- paste(make.col.names, ", ", "tkentry(.tableFrame, width='5', textvariable=", 
                    col.varname, ")", sep="")
            }
        eval(parse(text=paste("tkgrid(", make.col.names, ")", sep="")), envir=env)
        for (i in 1:nrows){   
            varname <- paste(".tab.", i, ".1", sep="") 
            assign(varname, tclVar("") , envir=env)
            row.varname <- paste(".rowname.", i, sep="")
            assign(row.varname, tclVar(i), envir=env)
            make.row <- paste("tkentry(.tableFrame, width='5', textvariable=",
                row.varname, ")", sep="")
            make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='5', textvariable=", 
                varname, ")", sep="")
            for (j in 2:ncols){
                varname <- paste(".tab.", i, ".", j, sep="")
                assign(varname, tclVar(""), envir=env)
                make.row <- paste(make.row, ", ", "tkentry(.tableFrame, width='5', textvariable=", 
                    varname, ")", sep="")
                }
            eval(parse(text=paste("tkgrid(", make.row, ")", sep="")), envir=env)
            }
        tkgrid(get(".tableFrame", envir=env), sticky="w")
        }
    rowColFrame <- tkframe(top)
    rowsValue <- tclVar("2")
    rowsSlider <- tkscale(rowColFrame, from=2, to=10, showvalue=FALSE, variable=rowsValue,
        resolution=1, orient="horizontal", command=setUpTable)
    rowsShow <- tklabel(rowColFrame, textvariable=rowsValue, width=2, justify="right")
    colsValue <- tclVar("2")
    colsSlider <- tkscale(rowColFrame, from=2, to=10, showvalue=FALSE, variable=colsValue,
        resolution=1, orient="horizontal", command=setUpTable)
    colsShow <- tklabel(rowColFrame, textvariable=colsValue, width=2, justify="right")
    onOK <- function() {
        rowPct <- tclvalue(rowPercentsVariable)
        colPct <- tclvalue(colPercentsVariable)
        totPct <- tclvalue(totPercentsVariable)

        nrows <- as.numeric(tclvalue(rowsValue))
        ncols <- as.numeric(tclvalue(colsValue))
        cell <- 0
        counts <- rep(NA, nrows*ncols)
        row.names <- rep("", nrows)
        col.names <- rep("", ncols)
        for (i in 1:nrows) row.names[i] <- 
            eval(parse(text=paste("tclvalue(", paste(".rowname.", i, sep=""),")", sep="")))
        for (j in 1:ncols) col.names[j] <- 
            eval(parse(text=paste("tclvalue(", paste(".colname.", j, sep=""),")", sep="")))
        for (i in 1:nrows){
            for (j in 1:ncols){
                cell <- cell+1
                varname <- paste(".tab.", i, ".", j, sep="")
                counts[cell] <- as.numeric(eval(parse(text=paste("tclvalue(", varname,")", sep=""))))
                }
            }
        counts <- na.omit(counts)
        if (length(counts) != nrows*ncols){
            errorCondition(recall=enterTable.HH, message=sprintf(gettextRcmdr("Number of valid entries (%d)\nnot equal to number of rows (%d) * number of columns (%d)."), length(counts), nrows, ncols))
            return()
            }
        if (length(unique(row.names)) != nrows){
            errorCondition(recall=enterTable.HH, message=gettextRcmdr("Row names are not unique."))
            return()
            }     
        if (length(unique(col.names)) != ncols){
            errorCondition(recall=enterTable.HH, message=gettextRcmdr("Column names are not unique."))
            return()
            }     
##        percents <- as.character(tclvalue(percentsVariable))
        chisq <- tclvalue(chisqTestVariable)
        chisqComp <- tclvalue(chisqCompVariable)
        chiComp <- tclvalue(chiCompVariable)
        expected <- tclvalue(expFreqVariable)
        fisher <- tclvalue(fisherTestVariable)
        closeDialog()

        command <- paste("matrix(c(", paste(counts, collapse=","), "), ", nrows, ", ", ncols,
            ", byrow=TRUE)", sep="")
        assign(".Table", justDoIt(command), envir=.GlobalEnv)
        logger(paste(".Table <- ", command, sep=""))
        command <- paste("c(",paste(paste("'", row.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("rownames(.Table) <- ", command, sep=""))
        logger(paste("rownames(.Table) <- ", command, sep=""))
        command <- paste("c(",paste(paste("'", col.names, "'", sep=""), collapse=", "), ")", sep="")
        justDoIt(paste("colnames(.Table) <- ", command, sep=""))
        logger(paste("colnames(.Table) <- ", command, sep=""))
        doItAndPrint(".Table  # Counts")

        if (rowPct == 1) doItAndPrint("rowPercents(.Table) # Row Percentages")
        if (colPct == 1) doItAndPrint("colPercents(.Table) # Column Percentages")        
        if (totPct == 1) doItAndPrint("totPercents2.HH(.Table) # Total Percentages")        

 
        if (chisq == 1) {
            command <- "chisq.test(.Table, correct=FALSE)"
            logger(paste(".Test <- ", command, sep=""))
            assign(".Test", justDoIt(command), envir=.GlobalEnv)
            doItAndPrint(".Test")
            if (chisqComp == 1) doItAndPrint("round(.Test$residuals^2, 2) # Chi-square Components")
            if (chiComp == 1) doItAndPrint("round(.Test$residuals, 2) # Chi Components (residuals)")
            if (expected == 1) doItAndPrint("round(.Test$expected, 2) # Expected Counts")
##            if (expected == 1) doItAndPrint(".Test$expected # Expected Counts")
            warnText <- NULL
            if (0 < (nlt1 <- sum(.Test$expected < 1))) warnText <- paste(nlt1,
                gettextRcmdr("expected frequencies are less than 1"))
            if (0 < (nlt5 <- sum(.Test$expected < 5))) warnText <- paste(warnText, "\n", nlt5,
                gettextRcmdr(" expected frequencies are less than 5"), sep="")
            if (!is.null(warnText)) Message(message=warnText,
                type="warning")
            logger("remove(.Test)") 
            remove(.Test, envir=.GlobalEnv) 
            }
        if (fisher == 1) doItAndPrint("fisher.test(.Table)")
        logger("remove(.Table)") 
        remove(.Table, envir=.GlobalEnv)                                                      
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="chisq.test")

    checkBoxes(frame="percentsFrame",
               boxes=c("rowPercents",
                 "colPercents",
                 "totPercents"), 
               initialValues=c("0","0","0"), 
               labels=gettextRcmdr(c(
                 "Row percentages",
                 "Column percentages",
                 "Total percentages")))
    
##     radioButtons(name="percents",
##                  buttons=c("rowPercents",
##                    "columnPercents",
##                    "nonePercents"),
##                  values=c("row", "column", "none"),
##                  initialValue="none",
##                  labels=gettextRcmdr(c
##                    ("Row percentages",
##                     "Column percentages",
##                     "No percentages")),
##                  title=gettextRcmdr("Compute Percentages"))
    
    checkBoxes(frame="testsFrame",
               boxes=c(
                 "chisqTest",
                 "chisqComp",
                 "chiComp",
                 "expFreq",
                 "fisherTest"),
               initialValues=c("1", "1", "0", "0", "0"),
               labels=gettextRcmdr(c(
                 "Chi-square test of independence",
                 "Print chi-square components",
                 "Print chi components (residuals)",
                 "Print expected frequencies",
                 "Fisher's exact test")))

##     checkBoxes(frame="testsFrame",
##                boxes=c("chisq", "expFreq", "fisher"),
##                initialValues=c("1", "0", "0"),                               
##                labels=gettextRcmdr(c("Chi-square test of independence",
##                  "Print expected frequencies",
##                  "Fisher's exact test")))
    
    tkgrid(tklabel(rowColFrame,
                   text=gettextRcmdr("Number of Rows:")),
           rowsSlider, rowsShow, sticky="w")
    tkgrid(tklabel(rowColFrame,
                   text=gettextRcmdr("Number of Columns:")),
           colsSlider, colsShow, sticky="w")
    tkgrid(rowColFrame, sticky="w")
    tkgrid(tklabel(top,
                   text=gettextRcmdr("Enter counts:"),
                   fg="blue"), sticky="w")
    tkgrid(outerTableFrame, sticky="w")
    tkgrid(tklabel(top, text=gettextRcmdr("Compute Percentages"), fg="blue"), sticky="w")
    tkgrid(percentsFrame, sticky="w")
    tkgrid(tklabel(top,
                   text=gettextRcmdr("Hypothesis Tests"),
                   fg="blue"), sticky="w")
    tkgrid(testsFrame, sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=7, columns=2)
    }

"lm.regsubsets" <-
function(object, model.number, ...) {
  sum.reg <- summary.regsubsets(object)
  dim.sum.reg <- dim(sum.reg$outmat)
  vars <- sum.reg$outmat[model.number,]
  rhs <- paste(names(vars)[vars=="*"], collapse="+")
  rhs <- parse(text=paste("~", rhs))[[1]][[2]]
  lm.call <- object$call
  lm.call[[1]] <- as.name("lm")
  lm.call[[2]][[3]] <- rhs
  llmc <- length(lm.call)
  if (llmc > 3) for (i in 4:llmc) lm.call[[i]] <- NULL
  lm.call[[1]] <- as.name("lm")
  eval.parent(lm.call, 2)
}

normalHypothesesPlot <- function(){
    initializeDialog(title=gettextRcmdr("Normal Distribution"))
    muVar <- tclVar("0")
    muEntry <- tkentry(top, width="6", textvariable=muVar)
    sigmaVar <- tclVar("1")
    sigmaEntry <- tkentry(top, width="6", textvariable=sigmaVar)
    nVar <- tclVar("")
    nEntry <- tkentry(top, width="6", textvariable=nVar)
    criticalLowVar <- tclVar("")
    criticalLowEntry <- tkentry(top, width="6", textvariable=criticalLowVar)
    criticalHighVar <- tclVar("1.645")
    criticalHighEntry <- tkentry(top, width="6", textvariable=criticalHighVar)
    obsValueVar <- tclVar("")
    obsValueEntry <- tkentry(top, width="6", textvariable=obsValueVar)
    muAltVar <- tclVar("")
    muAltEntry <- tkentry(top, width="6", textvariable=muAltVar)
    ymaxVar <- tclVar("")
    ymaxEntry <- tkentry(top, width="6", textvariable=ymaxVar)
    onOK <- function(){
      closeDialog()
      mu <- as.numeric(tclvalue(muVar))
      sigma <- as.numeric(tclvalue(sigmaVar))
      n <- as.numeric(tclvalue(nVar))
      criticalLow <- as.numeric(tclvalue(criticalLowVar))
      criticalHigh <- as.numeric(tclvalue(criticalHighVar))
      obsValue <- as.numeric(tclvalue(obsValueVar))
      muAlt <- as.numeric(tclvalue(muAltVar))
      ymax <- as.numeric(tclvalue(ymaxVar))
      
      ## norm.setup(mean=mu, se=sigma)
      ## norm.curve(mean=mu, se=sigma, critical=mean+critical*se, shade="right", col="black", axis.name="z")
      
      command <- "old.par <- par(oma=c(4,0,2,5), mar=c(7,7,4,2)+.1)"
      justDoIt(command)
      logger(command)
      
      if (is.na(sigma)) sigma <- 1
      if (is.na(mu)) mu <- 0
      if (is.na(n) && sigma==1)
        command.se <- ""
      if (is.na(n)) n <- 1
      if (!(exists("command.se") && command.se == ""))
        command.se <- paste(", se=",  sigma, "/sqrt(", n, ")", sep="")
      se <- sigma/sqrt(n)
      
      command.xlim <-
        if (mu !=0 || sigma != 1 || !is.na(muAlt) || !is.na(obsValue)) {
          xlim <-
            if (is.na(muAlt) && is.na(obsValue))
              mu + c(-1,1) * 2.5 * se
            else
              c(min(mu, muAlt, obsValue, na.rm=TRUE) - 2.5 * se,
                max(mu, muAlt, obsValue, na.rm=TRUE) + 2.5 * se)
          paste(", xlim=c(", xlim[1], ", ", xlim[2], ")", sep="")
        } else ""
      
      command.ylim <-
        if (!is.na(ymax)) paste(", ylim=c(0, ", ymax, ")", sep="")
        else ""
      
      command <- paste("norm.setup(mean=", mu,
                       command.se,
                       command.xlim,
                       command.ylim,
                       ")", sep="")
      justDoIt(command)
      logger(command)
      
      critical <- c(criticalLow, criticalHigh)
      critical <- critical[!is.na(critical)]
      shade <- "right"
      if (length(critical)==0) {
        critical <- mu + 20*se  ## big number, out of range of plot
      }
      else
        if (length(critical)==2) {
          critical <- paste("c(", critical[1], ",", critical[2], ")")
          shade="outside"
        }
      command.critical <- paste(", critical=", mu, "+", critical, "*", se)
      
      ## Plot Alternate Hypothesis first
      if (!is.na(muAlt)) {
        command <- paste("norm.curve(mean=", muAlt,
                         command.se,
                         command.critical,
                         ", shade='", "left", "', col='red', axis.name='z1'", ")", sep="")
        doItAndPrint(command)
      }
      
      ## Plot Null Hypothesis second
      command <- paste("norm.curve(mean=", mu,
                       command.se,
                       command.critical,
                       ", shade='", shade, "', col='black', axis.name='z'", ")", sep="")
      doItAndPrint(command)

      ## Observed Value
      if (!is.na(obsValue)) {
        command <- paste("norm.observed(", obsValue, ")", sep="")
        doItAndPrint(command)
      }

      command <- "par(old.par)"
      justDoIt(command)
      logger(command)
      
      tkfocus(CommanderWindow())
    }
    OKCancelHelp(helpSubject="dnorm")
    tkgrid(tklabel(top, text=gettextRcmdr("mu (mean)")), muEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("sigma (standard deviation)")), sigmaEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("n (sample size)")), nEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("z_alpha (critical value)")), criticalLowEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("-z_alpha (critical value)")), criticalHighEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("Observed Value")), obsValueEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("mu (Alternate Hypothesis)")), muAltEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("ymax (right-hand side)")), ymaxEntry, sticky="e")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    tkgrid.configure(muEntry, sticky="w")
    tkgrid.configure(sigmaEntry, sticky="w")
    tkgrid.configure(nEntry, sticky="w")
    tkgrid.configure(criticalLowEntry, sticky="w")
    tkgrid.configure(criticalHighEntry, sticky="w")
    tkgrid.configure(obsValueEntry, sticky="w")
    tkgrid.configure(muAltEntry, sticky="w")
    tkgrid.configure(ymaxEntry, sticky="w")
    dialogSuffix(rows=5, columns=2, focus=muEntry)
    }
"scatter3d.HH" <-
function (x, y, z,
            xlab = deparse(substitute(x)),
            ylab = deparse(substitute(y)), 
            zlab = deparse(substitute(z)),
            revolutions = 0,
            bg.col = c("white", "black"),
            axis.col = if (bg.col == "white") "black" else "white", 
            surface.col = c("blue", "green", "orange", "magenta", "cyan", 
              "red", "yellow", "gray"),
            neg.res.col = "red", pos.res.col = "green", 
            point.col = "yellow", text.col = axis.col,
            grid.col = if (bg.col == "white") "black" else "gray",
            fogtype = c("exp2", "linear", "exp", "none"),
            residuals = (length(fit) == 1), surface = TRUE, 
            grid = TRUE, grid.lines = 26, df.smooth = NULL, df.additive = NULL, 
            sphere.size = 1, threshold = 0.01,
            speed = 1, fov = 60, fit = "linear", 
            groups = NULL, parallel = TRUE, model.summary = FALSE,
            squares=FALSE, square.color="gray", coef.ratio=1)
{
    require(rgl)
    require(mgcv)
    summaries <- list()
    if ((!is.null(groups)) && (nlevels(groups) > length(surface.col))) 
        stop(sprintf(gettextRcmdr("Number of groups (%d) exceeds number of colors (%d)."), 
            nlevels(groups), length(surface.col)))
    if ((!is.null(groups)) && (!is.factor(groups))) 
        stop(gettextRcmdr("groups variable must be a factor."))
    bg.col <- match.arg(bg.col)
    fogtype <- match.arg(fogtype)
    if ((length(fit) > 1) && residuals && surface) 
        stop(gettextRcmdr("cannot plot both multiple surfaces and residuals"))
    xlab
    ylab
    zlab
    rgl.clear()
    rgl.viewpoint(fov = fov)
    rgl.bg(col = bg.col, fogtype = fogtype)
    valid <- if (is.null(groups)) 
        complete.cases(x, y, z)
    else complete.cases(x, y, z, groups)
    x <- x[valid]
    y <- y[valid]
    z <- z[valid]
    if (!is.null(groups)) 
        groups <- groups[valid]
    x <- (x - min(x))/(max(x) - min(x))
    y <- (y - min(y))/(max(y) - min(y))
    z <- (z - min(z))/(max(z) - min(z))
    size <- sphere.size * ((100/length(x))^(1/3)) * 0.015
    if (is.null(groups)) {
        if (size > threshold) 
            rgl.spheres(x, y, z, color = point.col, radius = size)
        else rgl.points(x, y, z, color = point.col)
    }
    else {
        if (size > threshold) 
            rgl.spheres(x, y, z, color = surface.col[as.numeric(groups)], 
                radius = size)
        else rgl.points(x, y, z, color = surface.col[as.numeric(groups)])
    }
    rgl.lines(c(0, 1), c(0, 0), c(0, 0), color = axis.col)
    rgl.lines(c(0, 0), c(0, 1), c(0, 0), color = axis.col)
    rgl.lines(c(0, 0), c(0, 0), c(0, 1), color = axis.col)
    rgl.texts(1, 0, 0, xlab, adj = 1, color = text.col)
    rgl.texts(0, 1, 0, ylab, adj = 1, color = text.col)
    rgl.texts(0, 0, 1, zlab, adj = 1, color = text.col)
    if (surface) {
        vals <- seq(0, 1, length = grid.lines)
        dat <- expand.grid(x = vals, z = vals)
        for (i in 1:length(fit)) {
            f <- match.arg(fit[i], c("linear", "quadratic", "smooth", 
                "additive"))
            if (is.null(groups)) {
                mod <-
                  switch(f,
                         linear = lm(y ~ x + z),
                         quadratic = lm(y ~ (x + z)^2 + I(x^2) + I(z^2)),
                         smooth =
                         if (is.null(df.smooth)) gam(y ~ s(x, z))
                         else gam(y ~ s(x, z, fx = TRUE, k = df.smooth)), 
                         additive =
                         if (is.null(df.additive)) gam(y ~ s(x) + s(z))
                         else gam(y ~ s(x, fx=TRUE, k=df.additive[1]+1)
                                  + s(z, fx=TRUE, k=(rev(df.additive+1)[1]+1))))
                if (model.summary) 
                  summaries[[f]] <- summary(mod)
                if (coef.ratio != 1) ## change the coefficients
                  mod$coefficients <- coef.ratio * mod$coefficients
                yhat <- matrix(predict(mod, newdata = dat), grid.lines, 
                  grid.lines)
                rgl.surface(vals, vals, yhat, color = surface.col[i], 
                  alpha = 0.5, lit = FALSE)
                if (grid) 
                  rgl.surface(vals, vals, yhat, color = grid.col, 
                    alpha = 0.5, lit = FALSE, front = "lines", 
                    back = "lines")
                if (residuals) {
                  n <- length(y)
                  if (coef.ratio != 1)
                    fitted <- predict(mod) ## new calculation
                  else
                    fitted <- fitted(mod) ## component of mod object
                  colors <- ifelse(residuals(mod) > 0, pos.res.col, 
                    neg.res.col)
                  if (squares)
                    rgl.quads(as.vector(rbind(x, x,
                                              x+(y-fitted), x+(y-fitted))),
                              as.vector(rbind(y, fitted, fitted, y)),
                              as.vector(rbind(z, z, z, z)),
                              color = square.color)
                  rgl.lines(as.vector(rbind(x, x)),
                            as.vector(rbind(y, fitted)),
                            as.vector(rbind(z, z)),
                            color = as.vector(rbind(colors, colors)))
                }
            }
            else {
                if (parallel) {
                  mod <- switch(f, linear = lm(y ~ x + z + groups), 
                    quadratic = lm(y ~ (x + z)^2 + I(x^2) + I(z^2) + 
                      groups), smooth = if (is.null(df.smooth)) 
                      gam(y ~ s(x, z) + groups)
                    else gam(y ~ s(x, z, fx = TRUE, k = df.smooth) + 
                      groups), additive = if (is.null(df.additive)) 
                      gam(y ~ s(x) + s(z) + groups)
                    else gam(y ~ s(x, fx = TRUE, k = df.additive[1] + 
                      1) + s(z, fx = TRUE, k = (rev(df.additive + 
                      1)[1] + 1)) + groups))
                  if (model.summary) 
                    summaries[[f]] <- summary(mod)
                  levs <- levels(groups)
                  for (j in 1:length(levs)) {
                    group <- levs[j]
                    select.obs <- groups == group
                    yhat <- matrix(predict(mod, newdata = cbind(dat, 
                      groups = group)), grid.lines, grid.lines)
                    rgl.surface(vals, vals, yhat, color = surface.col[j], 
                      alpha = 0.5, lit = FALSE)
                    if (grid) 
                      rgl.surface(vals, vals, yhat, color = grid.col, 
                        alpha = 0.5, lit = FALSE, front = "lines", 
                        back = "lines")
                    rgl.texts(0, predict(mod, newdata = data.frame(x = 0, 
                      z = 0, groups = group)), 0, paste(group, 
                      " "), adj = 1, color = surface.col[j])
                    if (residuals) {
                      yy <- y[select.obs]
                      xx <- x[select.obs]
                      zz <- z[select.obs]
                      fitted <- fitted(mod)[select.obs]
                      rgl.lines(as.vector(rbind(xx, xx)), as.vector(rbind(yy, 
                        fitted)), as.vector(rbind(zz, zz)), col = surface.col[j])
                    }
                  }
                }
                else {
                  levs <- levels(groups)
                  for (j in 1:length(levs)) {
                    group <- levs[j]
                    select.obs <- groups == group
                    mod <- switch(f, linear = lm(y ~ x + z, subset = select.obs), 
                      quadratic = lm(y ~ (x + z)^2 + I(x^2) + 
                        I(z^2), subset = select.obs), smooth = if (is.null(df.smooth)) 
                        gam(y ~ s(x, z), subset = select.obs)
                      else gam(y ~ s(x, z, fx = TRUE, k = df.smooth), 
                        subset = select.obs), additive = if (is.null(df.additive)) 
                        gam(y ~ s(x) + s(z), subset = select.obs)
                      else gam(y ~ s(x, fx = TRUE, k = df.additive[1] + 
                        1) + s(z, fx = TRUE, k = (rev(df.additive + 
                        1)[1] + 1)), subset = select.obs))
                    if (model.summary) 
                      summaries[[paste(f, ".", group, sep = "")]] <- summary(mod)
                    yhat <- matrix(predict(mod, newdata = dat), 
                      grid.lines, grid.lines)
                    rgl.surface(vals, vals, yhat, color = surface.col[j], 
                      alpha = 0.5, lit = FALSE)
                    rgl.surface(vals, vals, yhat, color = grid.col, 
                      alpha = 0.5, lit = FALSE, front = "lines", 
                      back = "lines")
                    rgl.texts(0, predict(mod, newdata = data.frame(x = 0, 
                      z = 0, groups = group)), 0, paste(group, 
                      " "), adj = 1, color = surface.col[j])
                    if (residuals) {
                      yy <- y[select.obs]
                      xx <- x[select.obs]
                      zz <- z[select.obs]
                      fitted <- fitted(mod)
                      rgl.lines(as.vector(rbind(xx, xx)), as.vector(rbind(yy, 
                        fitted)), as.vector(rbind(zz, zz)), col = surface.col[j])
                    }
                  }
                }
            }
        }
    }
    if (revolutions > 0) {
        for (i in 1:revolutions) {
            for (angle in seq(1, 360, length = 360/speed)) rgl.viewpoint(-angle, 
                fov = fov)
        }
    }
    if (model.summary) 
        return(summaries)
    else return(invisible(NULL))
}

"scatterPlotMatrix.HH" <-
function(){
    require("car")
    initializeDialog(title=gettextRcmdr("Scatterplot Matrix"))
    variablesBox <- variableListBox(top, Numeric(), title=gettextRcmdr("Select variables (three or more)"),
        selectmode="multiple", initialSelection=NULL)
    checkBoxes(frame="optionsFrame", boxes=c("lsLine", "smoothLine"), initialValues=rep(1,1),
        labels=gettextRcmdr(c("Least-squares lines", "Smooth lines")))
    sliderValue <- tclVar("50")
    slider <- tkscale(optionsFrame, from=0, to=100, showvalue=TRUE, variable=sliderValue,
        resolution=5, orient="horizontal")
    radioButtons(name="diagonal", buttons=c("density", "histogram", "boxplot", "oned", "qqplot", "none"),
        labels=gettextRcmdr(c("Density plots", "Histograms", "Boxplots", "One-dimensional scatterplots", "Normal QQ plots", "Nothing (empty)")),
        title=gettextRcmdr("On Diagonal"))
    subsetBox()
    onOK <- function(){
        variables <- getSelection(variablesBox)
        closeDialog()
        if (length(variables) < 3) {
            errorCondition(recall=scatterPlotMatrix.HH, message=gettextRcmdr("Fewer than 3 variable selected."))
            return()
            }
        line <- if("1" == tclvalue(lsLineVariable)) "lm" else "FALSE"
        smooth <- as.character("1" == tclvalue(smoothLineVariable))
        span <- as.numeric(tclvalue(sliderValue))
        diag <- as.character(tclvalue(diagonalVariable))
        subset <- tclvalue(subsetVariable)
        subset <- if (trim.blanks(subset) == gettextRcmdr("<all valid cases>")) "" 
            else paste(", subset=", subset, sep="")
        .activeDataSet <- ActiveDataSet()
        if (.groups == FALSE) {
          command <- paste("scatterplot.matrix(~", paste(variables, collapse="+"),
                           ", reg.line=", line, ", smooth=", smooth,
                           ", span=", span/100, ", diagonal = '", diag,
                           "', data=", .activeDataSet, subset,
                           ", row1attop= FALSE",
                           ")", sep="")
          logger(command)
          justDoIt(command)
        }
        else {
          command <- paste("scatterplot.matrix(~", paste(variables, collapse="+")," | ", .groups,
                           ", reg.line=", line, ", smooth=", smooth,
                           ", span=", span/100, ", diagonal= '", diag,
                           "', by.groups=", .linesByGroup,
                           ", data=", .activeDataSet, subset,
                           ", row1attop= FALSE",
                           ")", sep="")
          logger(command)
          justDoIt(command)
        }
        activateMenus()
        tkfocus(CommanderWindow())
        }
    groupsBox(scatterPlot, plotLinesByGroup=TRUE)
    OKCancelHelp(helpSubject="scatterplot.matrix")
    tkgrid(getFrame(variablesBox), sticky="nw")    
    tkgrid(optionsFrame, sticky="w")
    tkgrid(diagonalFrame, sticky="w")
    tkgrid(subsetFrame, sticky="w")
    tkgrid(groupsFrame, sticky="w")
    tkgrid(buttonsFrame, columnspan=2, sticky="w")
    dialogSuffix(rows=6, columns=2)
    }

tHypothesesPlot <- function(){
    initializeDialog(title=gettextRcmdr("t Distribution"))
    muVar <- tclVar("0")
    muEntry <- tkentry(top, width="6", textvariable=muVar)
    sigmaVar <- tclVar("1")
    sigmaEntry <- tkentry(top, width="6", textvariable=sigmaVar)
    degfreeVar <- tclVar("")
    degfreeEntry <- tkentry(top, width="6", textvariable=degfreeVar)
    nVar <- tclVar("")
    nEntry <- tkentry(top, width="6", textvariable=nVar)
    criticalLowVar <- tclVar("")
    criticalLowEntry <- tkentry(top, width="6", textvariable=criticalLowVar)
    criticalHighVar <- tclVar(".05")
    criticalHighEntry <- tkentry(top, width="6", textvariable=criticalHighVar)
    obsValueVar <- tclVar("")
    obsValueEntry <- tkentry(top, width="6", textvariable=obsValueVar)
    muAltVar <- tclVar("")
    muAltEntry <- tkentry(top, width="6", textvariable=muAltVar)
    ymaxVar <- tclVar("")
    ymaxEntry <- tkentry(top, width="6", textvariable=ymaxVar)
    onOK <- function(){
      closeDialog()
      mu <- as.numeric(tclvalue(muVar))
      sigma <- as.numeric(tclvalue(sigmaVar))
      degfree <- as.numeric(tclvalue(degfreeVar))
      n <- as.numeric(tclvalue(nVar))
      criticalLow <- as.numeric(tclvalue(criticalLowVar))
      criticalHigh <- 1-as.numeric(tclvalue(criticalHighVar))
      obsValue <- as.numeric(tclvalue(obsValueVar))
      muAlt <- as.numeric(tclvalue(muAltVar))
      ymax <- as.numeric(tclvalue(ymaxVar))
      
      ## norm.setup(mean=mu, se=sigma)
      ## norm.curve(mean=mu, se=sigma, critical=mean+critical*se, shade="right", col="black", axis.name="z")
      
      command <- "old.par <- par(oma=c(4,0,2,5), mar=c(7,7,4,2)+.1)"
      justDoIt(command)
      logger(command)
      
      if (is.na(sigma)) sigma <- 1
      if (is.na(mu)) mu <- 0
      if (is.na(n) && sigma==1)
        command.se <- ""
      if (is.na(n)) n <- 1
      if (!(exists("command.se") && command.se == ""))
        command.se <- paste(", se=",  sigma, "/sqrt(", n, ")", sep="")
      se <- sigma/sqrt(n)
      
      command.xlim <-
        if (mu !=0 || sigma != 1 || !is.na(muAlt) || !is.na(obsValue)) {
          xlim <-
            if (is.na(muAlt) && is.na(obsValue))
              mu + c(-1,1) * 2.5 * se
            else
              c(min(mu, muAlt, obsValue, na.rm=TRUE) - 2.5 * se,
                max(mu, muAlt, obsValue, na.rm=TRUE) + 2.5 * se)
          paste(", xlim=c(", xlim[1], ", ", xlim[2], ")", sep="")
        } else ""
      
      command.ylim <-
        if (!is.na(ymax)) paste(", ylim=c(0, ", ymax, ")", sep="")
        else ""
      
      command <- paste("norm.setup(mean=", mu,
                       command.se,
                       command.xlim,
                       command.ylim,
                       sep="")
      if (!is.na(degfree))
        command <- paste(command, ", df=", degfree, sep="")
      else
        degfree <- Inf
      command <- paste(command, ")")
      justDoIt(command)
      logger(command)
      
      critical <- c(criticalLow, criticalHigh)
      critical <- critical[!is.na(critical)]
      critical <- qt(critical, degfree)
      shade <- "right"
      
      if (length(critical)==2) {
        critical <- paste("c(", critical[1], ",", critical[2], ")")
        shade="outside"
      }

      if (length(critical)==0) {
        critical <- mu + 20*se  ## big number, out of range of plot
      }

      command.critical <- paste(", critical=", mu, "+", critical, "*", se)

      if (is.na(degfree) || degfree==Inf)
        axis.name <- "z"
      else
        axis.name <- "t"
      
      ## Plot Alternate Hypothesis first
      if (!is.na(muAlt)) {
        command <- paste("norm.curve(mean=", muAlt,
                         command.se,
                         command.critical,
                         ", shade='", "left",
                         "', col='red', axis.name='", axis.name, "1'",
                         sep="")
        if (!(is.na(degfree) || degfree==Inf))
          command <- paste(command, ", df=", degfree, sep="")
        command <- paste(command, ")")
        doItAndPrint(command)
      }
      
      ## Plot Null Hypothesis second
      command <- paste("norm.curve(mean=", mu,
                       command.se,
                       command.critical,
                       ", shade='", shade,
                       "', col='black', axis.name='", axis.name, "'",
                       sep="")
      if (!is.na(degfree))
        command <- paste(command, ", df=", degfree, sep="")
      command <- paste(command, ")")
      doItAndPrint(command)

      ## Observed Value
      if (!is.na(obsValue)) {
        obs.T <- paste("round((", obsValue, " - ", mu, ")/(",  sigma, "/sqrt(", n, ")", "), 3)")
        command <- paste("norm.observed(", obsValue, ", ", obs.T, ")", sep="")
        doItAndPrint(command)
      }

      command <- "par(old.par)"
      justDoIt(command)
      logger(command)
      
      tkfocus(CommanderWindow())
    }
    OKCancelHelp(helpSubject="dnorm")
    tkgrid(tklabel(top, text=gettextRcmdr("mu (mean)")), muEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("s (standard deviation)")), sigmaEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("df (degrees of freedom)")), degfreeEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("n (sample size)")), nEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("left alpha")), criticalLowEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("right alpha")), criticalHighEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("Observed Value")), obsValueEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("mu (Alternate Hypothesis)")), muAltEntry, sticky="e")
    tkgrid(tklabel(top, text=gettextRcmdr("ymax (right-hand side)")), ymaxEntry, sticky="e")
    tkgrid(buttonsFrame, columnspan=1, sticky="w")
    tkgrid.configure(muEntry, sticky="w")
    tkgrid.configure(sigmaEntry, sticky="w")
    tkgrid.configure(nEntry, sticky="w")
    tkgrid.configure(criticalLowEntry, sticky="w")
    tkgrid.configure(criticalHighEntry, sticky="w")
    tkgrid.configure(obsValueEntry, sticky="w")
    tkgrid.configure(muAltEntry, sticky="w")
    tkgrid.configure(ymaxEntry, sticky="w")
    dialogSuffix(rows=8, columns=1, focus=muEntry)
    }
"twoWayTable.HH" <-
function(){
    require("abind")
    initializeDialog(title=gettextRcmdr("Two-Way Table"))
    variablesFrame <- tkframe(top)
    .factors <- Factors()
    rowBox <- variableListBox(variablesFrame, .factors, title=gettextRcmdr("Row variable (pick one)"))
    columnBox <- variableListBox(variablesFrame, .factors, title=gettextRcmdr("Column variable (pick one)"))
    subsetBox()
    onOK <- function() {
        row <- getSelection(rowBox)
        column <- getSelection(columnBox)
        if (length(row) == 0 || length(column) == 0){
            errorCondition(recall=twoWayTable.HH, message=gettextRcmdr("You must select two variables."))
            return()
            }
        if (row == column) {
            errorCondition(recall=twoWayTable.HH, message=gettextRcmdr("Row and column variables are the same."))
            return()
            }        

        rowPct <- tclvalue(rowPercentsVariable)
        colPct <- tclvalue(colPercentsVariable)
        totPct <- tclvalue(totPercentsVariable)
##        percents <- as.character(tclvalue(percentsVariable))
        chisq <- tclvalue(chisqTestVariable)
        chisqComp <- tclvalue(chisqCompVariable)
        chiComp <- tclvalue(chiCompVariable)
        expected <- tclvalue(expFreqVariable)
        fisher <- tclvalue(fisherTestVariable)

        subset <- tclvalue(subsetVariable)
        subset <- if (trim.blanks(subset) == gettextRcmdr("<all valid cases>")) "" 
            else paste(", subset=", subset, sep="")
        closeDialog()

        command <- paste("xtabs(~", row, "+", column, ", data=", ActiveDataSet(),
            subset, ")", sep="")
        logger(paste(".Table <- ", command, sep=""))
        assign(".Table", justDoIt(command), envir=.GlobalEnv)
        doItAndPrint(".Table")

        if (rowPct == 1) doItAndPrint("rowPercents(.Table) # Row Percentages")
        if (colPct == 1) doItAndPrint("colPercents(.Table) # Column Percentages")        
        if (totPct == 1) doItAndPrint("totPercents2.HH(.Table) # Total Percentages")        

        if (chisq == 1) {
            command <- "chisq.test(.Table, correct=FALSE)"
            logger(paste(".Test <- ", command, sep=""))
            assign(".Test", justDoIt(command), envir=.GlobalEnv)
            doItAndPrint(".Test")
            if (chisqComp == 1) doItAndPrint("round(.Test$residuals^2, 2) # Chi-square Components")
            if (chiComp == 1) doItAndPrint("round(.Test$residuals, 2) # Chi Components (residuals)")
            if (expected == 1) doItAndPrint("round(.Test$expected, 2) # Expected Counts")
##            if (expected == 1) doItAndPrint(".Test$expected # Expected Counts")
            warnText <- NULL
            if (0 < (nlt1 <- sum(.Test$expected < 1))) warnText <- paste(nlt1,
                gettextRcmdr("expected frequencies are less than 1"))
            if (0 < (nlt5 <- sum(.Test$expected < 5))) warnText <- paste(warnText, "\n", nlt5,
                gettextRcmdr(" expected frequencies are less than 5"), sep="")
            if (!is.null(warnText)) Message(message=warnText,
                type="warning")
            logger("remove(.Test)") 
            remove(.Test, envir=.GlobalEnv) 
            }
        if (fisher == 1) doItAndPrint("fisher.test(.Table)")
        logger("remove(.Table)") 
        remove(.Table, envir=.GlobalEnv)                                                      
        tkfocus(CommanderWindow())
        }
    OKCancelHelp(helpSubject="xtabs")

    checkBoxes(frame="percentsFrame",
               boxes=c("rowPercents",
                 "colPercents",
                 "totPercents"), 
               initialValues=c("0","0","0"), 
               labels=gettextRcmdr(c(
                 "Row percentages",
                 "Column percentages",
                 "Total percentages")))
    
##   radioButtons(name="percents",
##                buttons=c("rowPercents",
##                  "columnPercents",
##                  "nonePercents"), 
##                values=c("row", "column", "none"),
##                initialValue="none", 
##                labels=gettextRcmdr(c
##                  ("Row percentages",
##                   "Column percentages",
##                   "No percentages")),
##                title=gettextRcmdr("Compute Percentages"))
    
    checkBoxes(frame="testsFrame",
               boxes=c(
                 "chisqTest",
                 "chisqComp",
                 "chiComp",
                 "expFreq",
                 "fisherTest"),
               initialValues=c("1", "1", "0", "0", "0"),
               labels=gettextRcmdr(c(
                 "Chi-square test of independence",
                 "Print chi-square components",
                 "Print chi components (residuals)",
                 "Print expected frequencies",
                 "Fisher's exact test")))

##     checkBoxes(frame="testsFrame",
##                boxes=c("chisqTest", "expFreq", "fisherTest"), initialValues=c("1", "0", "0"),
##                labels=gettextRcmdr(
##                  c("Chi-square test of independence",
##                    "Print expected frequencies",
##                    "Fisher's exact test")))

    tkgrid(getFrame(rowBox),
           tklabel(variablesFrame, text="    "),
           getFrame(columnBox), sticky="nw")
    tkgrid(variablesFrame, sticky="w")
    tkgrid(tklabel(top, text=gettextRcmdr("Compute Percentages"), fg="blue"), sticky="w")
    tkgrid(percentsFrame, sticky="w")
    tkgrid(tklabel(top,
                   text=gettextRcmdr("Hypothesis Tests"),
                   fg="blue"),
           sticky="w")
    tkgrid(testsFrame, sticky="w")
    tkgrid(subsetFrame, sticky="w")
    tkgrid(buttonsFrame, sticky="w")
    dialogSuffix(rows=6, columns=1)
  }

.First.lib <- function(libname, pkgname){
  if (!interactive()) return()
  Rcmdr <- options()$Rcmdr
  plugins <- Rcmdr$plugins
  if ((!pkgname %in% plugins) && !getRcmdr("autoRestart")) {
    Rcmdr$plugins <- c(plugins, pkgname)
    options(Rcmdr=Rcmdr)
    closeCommander(ask=FALSE, ask.save=TRUE)
    Commander()
  }
}

